@@ -81,11 +81,12 @@ class GAN:
     The GAN is made from a generator and a discriminator
     """
 
-    def __init__(self, gen='', dis='', npix=128, train_disc=False, noiselength=100, Adam_lr=0.0001, Adam_beta_1=0.91, resetOpt=False):
+    def __init__(self, gen='', dis='', npix=128, train_disc=False, noiselength=100, Adam_lr=0.0001, Adam_beta_1=0.91, resetOpt=False, amsgrad=False):
         self.resetOpt = resetOpt
         self.Adam_lr = Adam_lr
         self.Adam_beta_1 = Adam_beta_1
-        self.opt = self.getOptimizer(self.Adam_lr, self.Adam_beta_1)
+        self.amsgrad = amsgrad
+        self.opt = self.getOptimizer(self.Adam_lr, self.Adam_beta_1, amsgrad = self.amsgrad)
         self.train_disc = train_disc
         self.noiselength = noiselength
         if gen != '' and dis != '':
@@ -101,8 +102,8 @@ class GAN:
 
 
     @staticmethod
-    def getOptimizer(lr, beta1, beta2=0.999, epsilon = 1e-7):
-        return Adam(learning_rate = lr, beta_1 = beta1, beta_2 = beta2, epsilon = epsilon, amsgrad=False)
+    def getOptimizer(lr, beta1, beta2=0.999, epsilon = 1e-7, amsgrad = False):
+        return Adam(learning_rate = lr, beta_1 = beta1, beta_2 = beta2, epsilon = epsilon, amsgrad=amsgrad)
 
     def read(self):
         '''
@@ -117,6 +118,11 @@ class GAN:
         self.gen = gen
         self.dis = dis
         self.npix = gen.output.shape[1]
+        # create copy of generator
+        gen_copy = tf.keras.models.clone_model(gen)
+        gen_copy.set_weights(gen.get_weights())
+        self.gen_init = gen_copy
+
 
 
     def create_generator(self, ReLU=0.25):
@@ -181,14 +187,20 @@ class GAN:
         gan: a compiled keras model where the generator is followed by the discriminator and the discriminator is not trainable
 
     """
-    def create_gan(self, train_disc = False):
+    def create_gan(self, train_disc = False, train_gen = True, reinit = False):
 
+        if reinit:
+            gen = self.gen_init
+        else:
+            gen = self.gen
         self.dis.trainable = train_disc
+        gen.trainable = train_gen
         gan_input = layers.Input(shape = (self.noiselength,))
-        x = self.gen(gan_input)
+        x = gen(gan_input)
         gan_output= self.dis(x)
         gan= Model(inputs=gan_input, outputs=[gan_output, x])
         gan.compile(loss='binary_crossentropy', optimizer = self.opt, metrics=["accuracy"])
+        self.gen = gen
         return gan
 
     '''
@@ -511,7 +523,7 @@ effect:
             warn(f'It may overwrite files if they already exist!')
 
         # copy the data file to the directory
-        shutil.copyfile(os.path.join(data_dir, data_files), os.path.join(os.getcwd(), 'OIData.fits'))
+        # shutil.copyfile(os.path.join(data_dir, data_files), os.path.join(os.getcwd(), 'OIData.fits'))
 
 
         # Creating dictionary with image recsontruction parameters
@@ -606,13 +618,22 @@ effect:
         self.ImgRec()
 
     def ImgRec(self ):
+        # Re init GAN
+        self.gan = self.create_gan(reinit = True)
+        self.gan.get_layer(index=1).set_weights(self.gen.get_weights())
+        if self.resetOpt == True:
+            opt = 'optimizers.'+self.opt._name
+            opt = eval(opt)
+            opt = opt.from_config(self.opt.get_config())
+            self.gan.compile(loss=[CrossEntropy, data_loss], optimizer= opt,loss_weights=[mu,1])
+
+        # get parameters
         params = self.params
         mu = params['mu']
         #Create data loss
         data_loss = self.set_dataloss()
-        # Update GAN
-        GeneratorCopy = tf.keras.models.clone_model(self.gen)
-        GeneratorCopy.set_weights(self.gen.get_weights())
+
+
         outdir = os.path.join(os.getcwd(),self.dir)
 
         Chi2s, DisLoss = [], []
@@ -622,15 +643,7 @@ effect:
         if self.diagnostics:
             print('#restart\tftot\tfdata\tfdiscriminator')
         for r in iteration:
-            #GeneratorCopy.set_weights(Generator.get_weights())
 
-            if self.gan != None:
-                self.gan.get_layer(index=1).set_weights(self.gen.get_weights())
-                if self.resetOpt == True:
-                    opt = 'optimizers.'+self.opt._name
-                    opt = eval(opt)
-                    opt = opt.from_config(self.opt.get_config())
-                    self.gan.compile(loss=[CrossEntropy, data_loss], optimizer= opt,loss_weights=[mu,1])
 
             # generating the noise vector for this restart
             noisevector = np.array([np.random.normal(0, 1, 100)])
